; LPC932IAPLIB
; In-Application-Programming Library for the Philips LPC932
; (C) Embedded Systems Academy 2003
; This library was developed for the Raisonance and Keil Compilers for interfacing
; IAP routines to C programs

; Version 1.00, Andrew Ayre, 24-Feb-2003
;  - first version

$NOMOD51

; module name
NAME    LPC932IAPLIB

; segments in this library
?BI?LPC932IAPLIB                           SEGMENT BIT
?DT?LPC932IAPLIB                           SEGMENT DATA
?PR?iap_lib?LPC932IAPLIB                   SEGMENT CODE
?PR?_iap_init?LPC932IAPLIB                 SEGMENT CODE
?PR?iap_read_version?LPC932IAPLIB          SEGMENT CODE
?PR?_iap_write_code?LPC932IAPLIB           SEGMENT CODE
?PR?_iap_erase?LPC932IAPLIB                SEGMENT CODE
?PR?_iap_read_sector_crc?LPC932IAPLIB      SEGMENT CODE
?PR?iap_read_global_crc?LPC932IAPLIB       SEGMENT CODE
?PR?_iap_read_code?LPC932IAPLIB            SEGMENT CODE
?PR?_iap_read?LPC932IAPLIB                 SEGMENT CODE
?PR?_iap_write?LPC932IAPLIB                SEGMENT CODE

; function names and global variables
	PUBLIC iap_status
        PUBLIC _iap_init
        PUBLIC iap_read_version
	PUBLIC _iap_write_code
	PUBLIC _iap_erase
	PUBLIC _iap_read_sector_crc
	PUBLIC iap_read_global_crc
	PUBLIC _iap_read_code
	PUBLIC _iap_read
	PUBLIC _iap_write

; SFRs and sbits required
EA      BIT     0AFH
ACC     DATA    0E0H
IE      DATA    0A8H
WFEED1  DATA    0C2H
WFEED2  DATA    0C3H
PCON    DATA    087H

; global variables (data memory)
        RSEG  ?BI?LPC932IAPLIB
?BI?LPC932IAPLIB?BASE:
	iap_watchdogfeed: DBIT   1               ; watchdog feed flag
        iap_intdisable:   DBIT   1               ; interrupt disable flag

	RSEG  ?DT?LPC932IAPLIB
?DT?LPC932IAPLIB?BASE:
	iap_status:       DS     1               ; status variable

; possible values for iap_status
IAP_OK             EQU  00H
IAP_INTERRUPTED    EQU  01H
IAP_SECURITY       EQU  02H
IAP_HIGHVOTAGE     EQU  04H

; possible values for erase type
IAP_PAGE           EQU  00H
IAP_SECTOR         EQU  01H

; possible values for byte name for iap_read and iap_write
IAP_UCFG1          EQU  00H
IAP_BOOTVECTOR     EQU  02H
IAP_STATUSBYTE     EQU  03H
IAP_SECURITYBYTE0  EQU  08H
IAP_SECURITYBYTE1  EQU  09H
IAP_SECURITYBYTE2  EQU  0AH
IAP_SECURITYBYTE3  EQU  0BH
IAP_SECURITYBYTE4  EQU  0CH
IAP_SECURITYBYTE5  EQU  0DH
IAP_SECURITYBYTE6  EQU  0EH
IAP_SECURITYBYTE7  EQU  0FH

; IAP entry points
; change PGM_MPT to 0FF03H for:
; LPC920,921,922,924,925,930,931
PGM_MTP            EQU  0FF00H
IAP_PROG           EQU  PGM_MTP + 05H
IAP_ID             EQU  PGM_MTP + 07H
IAP_WRITE          EQU  PGM_MTP + 09H
IAP_READ           EQU  PGM_MTP + 0BH
IAP_ERASE          EQU  PGM_MTP + 0DH
IAP_SECCRC         EQU  PGM_MTP + 0FH
IAP_GLBCRC         EQU  PGM_MTP + 11H
IAP_READCODE       EQU  PGM_MTP + 13H

; **********************************************************************
; function:    iap_init
; prototype:   void iap_init(unsigned char configuration);
; description: configures IAP library. must be called before any
;              other functions.
; WARNING:     enable brown out detect in UCFG1 before using
; globals:     always sets iap_status to IAP_OK
; registers:   A, R7
; **********************************************************************
        RSEG  ?PR?_iap_init?LPC932IAPLIB
_iap_init:
        ; configuration = R7
        MOV A,R7
        RRC A                                    ; rotate bit 0 into carry
        MOV iap_watchdogfeed,C                   ; store in flag
        RRC A                                    ; rotate bit 1 into carry
        MOV iap_intdisable,C                     ; store in flag
        ANL PCON,#0DCH                           ; enable brown-out power down
        MOV iap_status,#IAP_OK                   ; operation OK
	RET
; end of iap_init

; **********************************************************************
; name:        IAP routine library
; description: contains rountes for use by the IAP library. Includes:
;              watchdog feed - corrupts C
;              exit - corrupts R7
;              disable interrupts - corrupts R0
;              enable interrupts - uses R0
; **********************************************************************
        RSEG  ?PR?iap_lib?LPC932IAPLIB
iap_feedwd:
        JNB     iap_watchdogfeed,iap_feedwdexit
	MOV     C,EA                             ; preserve interrupt setting
	CLR     EA                               ; disable interrupts
	MOV     WFEED1,#0A5H                     ; feed watchdog
	MOV     WFEED2,#5AH
        MOV     EA,C                             ; restore interrupt setting
iap_feedwdexit:
        RET
iap_disableint:
        JNB     iap_intdisable,iap_disableintexit
        MOV     R0,IE                            ; preserve interrupt settings
        CLR     EA                               ; disable interrupts
iap_disableintexit:
        RET
iap_enableint:
        JNB     iap_intdisable,iap_enableintexit
        MOV     IE,R0                            ; restore interrupt settings
iap_enableintexit:
        RET
iap_exit:
	JC      iap_error                        ; if error go to error handler
	MOV     iap_status,#IAP_OK               ; operation OK
	RET                                      ; return with result
iap_error:
        MOV     iap_status,R7                    ; put error codes in iap_status
	MOV     R7,#00H                          ; result is zero
	RET

; end of iap_lib

; **********************************************************************
; function:    iap_read_version
; prototype:   unsigned char iap_read_version(void);
; description: returns IAP version
; globals:     always sets iap_status to IAP_OK
; registers:   R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?iap_read_version?LPC932IAPLIB
iap_read_version:
        ; return value = R7
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_ID                           ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
	MOV     iap_status,#IAP_OK               ; operation OK
	RET
; end of iap_read_version

; **********************************************************************
; function:    iap_write_code
; prototype:   void iap_write_code(unsigned char data *pbuffer,
;                                  unsigned int address,
;				   unsigned char length);
; description: programs code memory. address = page address, 
;              pbuffer = pointer to buffer containing data,
;              length = length of data (1 - 64)
; WARNING:     address may be any address however data written must fit
;              entirely in a 64 byte page.
; globals:     sets iap_status
; registers:   A, R3, R4, R5, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_write_code?LPC932IAPLIB
_iap_write_code:
        ; pbuffer = R7, address = R4:R5, length = R3
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_PROG                         ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_write_code

; **********************************************************************
; function:    iap_erase
; prototype:   void iap_erase(unsigned char type, unsigned int address);
; description: erases sector or page. address = page or sector address
;              type = IAP_PAGE or IAP_SECTOR
; globals:     sets iap_status
; registers:   A, R4, R5, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_erase?LPC932IAPLIB
_iap_erase:
        ; type = R7, address = R4:R5
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_ERASE                        ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_erase

; **********************************************************************
; function:    iap_read_sector_crc
; prototype:   unsigned long iap_read_sector_crc(unsigned int address);
; description: reads 32-bit CRC of a sector. address = sector address
; globals:     sets iap_status
; registers:   A, R4, R5, R6, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_read_sector_crc?LPC932IAPLIB
_iap_read_sector_crc:
        ; address = R6:R7, return value = R4:R5:R6:R7
        LCALL   iap_feedwd                       ; feed watchdog if needed
        MOV     A,R6                             ; put MSB of address in R7
	MOV     R7,A
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_SECCRC                       ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_read_sector_crc

; **********************************************************************
; function:    iap_read_global_crc
; prototype:   unsigned long iap_read_global_crc(void);
; description: reads 32-bit CRC of device.
; globals:     sets iap_status
; registers:   R4, R5, R6, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?iap_read_global_crc?LPC932IAPLIB
iap_read_global_crc:
        ; return value = R4:R5:R6:R7
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_GLBCRC                       ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_read_global_crc

; **********************************************************************
; function:    iap_read_code
; prototype:   unsigned char iap_read_code(unsigned int address);
; description: reads data from code memory. address = address to read
; globals:     sets iap_status
; registers:   A, R4, R5, R6, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_read_code?LPC932IAPLIB
_iap_read_code:
        ; address = R6:R7, return value = R7
        LCALL   iap_feedwd                       ; feed watchdog if needed
	MOV     A,R7                             ; put address in R4:R5
	MOV     R5,A
	MOV     A,R6
	MOV     R4,A
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_READCODE                     ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_read_code

; **********************************************************************
; function:    iap_read
; prototype:   unsigned char iap_read(unsigned char name);
; description: returns one of the configuration bytes. name = byte to read
;              one of: IAP_UCFG1, IAP_BOOTVECTOR, IAP_STATUSBYTTE
;              IAP_SECURITYBYTE0 - IAP_SECURITYBYTE7
; globals:     sets iap_status
; registers:   R7 
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_read?LPC932IAPLIB
_iap_read:
        ; name = R7, return value = R7
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_READ                         ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_read

; **********************************************************************
; function:    iap_write
; prototype:   void iap_write(unsigned char name, unsigned char value);
; description: programs one of the configuration bytes. name = byte to
;              write, one of: IAP_UCFG1, IAP_BOOTVECTOR, IAP_STATUSBYTTE
;              IAP_SECURITYBYTE0 - IAP_SECURITYBYTE7.
;              value = value to write to byte
; globals:     sets iap_status
; registers:   R5, R7
;              R0 if IAP_INTDISABLE
; **********************************************************************
        RSEG  ?PR?_iap_write?LPC932IAPLIB
_iap_write:
        ; name = R7, value = R5
        LCALL   iap_feedwd                       ; feed watchdog if needed
	LCALL   iap_disableint                   ; disable interrupts if needed
	LCALL   IAP_WRITE                        ; call iap routine
	LCALL   iap_enableint                    ; enable interrupts if needed
        LJMP    iap_exit                         ; put status in iap_status and exit
; end of iap_write_ucfg1

; iap library test code for simulator
; uncomment to use
;        CSEG    AT PGM_MTP
;        MOV     R7,#01H
;        CLR     C
;        ;SETB    C
;        RET
; end of iap library test code
        
        END

