// values for configuration of library
// use IAP_WATCHDOGFEED to feed the watchdog immediately
// before the IAP call. This allows watchdog to be used as long
// as the timeout is greater than 2ms
// if watchdog timeout is 2ms or less and safety mode is
// being used then increase timeout or disable watchdog in
// UCFG1.
// if watchdog timeout is 2ms or less and safety mode is
// not being used then disable watchdog or increase
// timeout before making IAP call, then restore settings
// after IAP call
// use IAP_INTDISABLE to disable interrupts for duration
// of IAP call.
// If interrupts are enabled and IAP_INTDISABLE is not used,
// check iap_status for interrupted flag

// NOTE: To use library, Brown-out detect must be enabled in UCFG1
// Make sure UCFG1.5 is set to 1 (progammed).
#define IAP_WATCHDOGFEED     0x01
#define IAP_INTDISABLE       0x02

// values to pass to iap_read and iap_write
#define IAP_UCFG1         0x00
#define IAP_BOOTVECTOR    0x02
#define IAP_STATUSBYTE    0x03
#define IAP_SECURITYBYTE0 0x08
#define IAP_SECURITYBYTE1 0x09
#define IAP_SECURITYBYTE2 0x0A
#define IAP_SECURITYBYTE3 0x0B
#define IAP_SECURITYBYTE4 0x0C
#define IAP_SECURITYBYTE5 0x0D
#define IAP_SECURITYBYTE6 0x0E
#define IAP_SECURITYBYTE7 0x0F

// values for status
// it is possible that more than one value (apart from
// IAP_OK) may be returned at once
#define IAP_OK          0x00
#define IAP_INTERRUPTED 0x01
#define IAP_SECURITY    0x02
#define IAP_ERROR       0x04
#define IAP_ABORT       0x08

// values for erase type
#define IAP_PAGE        0x00
#define IAP_SECTOR      0x01

// status
extern unsigned char iap_status;

extern void iap_init(unsigned char configuration);
extern unsigned char iap_read_version(void);
extern void iap_write_code(unsigned char data *pbuffer, unsigned int address, unsigned char length);
extern void iap_erase(unsigned char type, unsigned int address);
extern unsigned long iap_read_sector_crc(unsigned int address);
extern unsigned long iap_read_global_crc(void);
extern unsigned char iap_read_code(unsigned int address);
extern unsigned char iap_read(unsigned char name);
extern void iap_write(unsigned char name, unsigned char value);

